//Lsm303d.cpp - Library for LSM303D Accelerometer/Magnetometer.
//Copyright (c) 2016, Julie VK3FOWL and Joe VK3YSP
//Released under the GNU General Public License.
//For more information please visit http://www.sarcnet.org
//Reference: ST Datasheet: LSM303D Ultra-compact high-performance eCompass module: 3D accelerometer and 3D magnetometer
#include "Lsm303d.h"

//Constructor
void Lsm303d::begin() {
  reset();
}

//Public

void Lsm303d::reset() {
  Wire.begin();
  writeReg(LSM303D_ADDRESS, LSM303D_CTRL1, 0b01010111); //Acc output data rate = 50Hz all Acc axes enabled.
  writeReg(LSM303D_ADDRESS, LSM303D_CTRL2, 0b00000000); //Acc full scale = +/- 2g
  writeReg(LSM303D_ADDRESS, LSM303D_CTRL5, 0b01100100); //Mag output data rate = 6.25HzMag resolution = high;
  writeReg(LSM303D_ADDRESS, LSM303D_CTRL6, 0b00100000); //Mag full scale = +/- 4gauss
  writeReg(LSM303D_ADDRESS, LSM303D_CTRL7, 0b00000000); //Mag low power mode = Off. Mag sensor mode = Continuous-conversion
}

void Lsm303d::read() {
  reset();  //Reinitialise I2C and the sensor before each read as it is inclined to lock up
  readM();
  readG();
}

//Private

int Lsm303d::lpf(int value, int *last, float alpha) {
  //Low pass filter - Decrease alpha to increase damping factor
  int result = (alpha * value) + *last * (1 - alpha);
  *last = result;
  return result;
}

void Lsm303d::readG() {
  Wire.beginTransmission((byte)LSM303D_ADDRESS);
  Wire.write(LSM303D_OUT_X_L_A | 0x80);
  Wire.endTransmission();
  Wire.requestFrom((byte)LSM303D_ADDRESS, (byte)6);
  if (Wire.available() == 6) {
    byte  yl = Wire.read();
    byte  yh = Wire.read();
    byte  xl = Wire.read();
    byte  xh = Wire.read();
    byte  zl = Wire.read();
    byte  zh = Wire.read();
    gx = ((xh << 8) | xl);
    gy = -((yh << 8) | yl);
    gz = -((zh << 8) | zl);
    //gx = -((xh << 8) | xl);
    //gy = ((yh << 8) | yl);
    //gz = ((zh << 8) | zl);
    //Low pass filter the sensor data as it improves the calibration procedure
    gx = lpf(gx, &gxLast, alpha);
    gy = lpf(gy, &gyLast, alpha);
    gz = lpf(gz, &gzLast, alpha);
  }
}

void Lsm303d::readM() {
  Wire.beginTransmission((byte)LSM303D_ADDRESS);
  Wire.write(LSM303D_OUT_X_L_M | 0x80);
  Wire.endTransmission();
  Wire.requestFrom((byte)LSM303D_ADDRESS, (byte)6);
  if (Wire.available() == 6) {
    byte yl = Wire.read();
    byte yh = Wire.read();
    byte xl = Wire.read();
    byte xh = Wire.read();
    byte zl = Wire.read();
    byte zh = Wire.read();
    mx = -((xh << 8) | xl);
    my = ((yh << 8) | yl);
    mz = ((zh << 8) | zl);
    //Low pass filter the sensor data as it improves the calibration procedure
    mx = lpf(mx, &mxLast, alpha);
    my = lpf(my, &myLast, alpha);
    mz = lpf(mz, &mzLast, alpha);
  }
}

void Lsm303d::writeReg(byte address, byte reg, byte value) {
  Wire.beginTransmission(address);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission();
}

byte Lsm303d::readReg(byte address, byte reg) {
  byte value;
  Wire.beginTransmission(address);
  Wire.write(reg);
  Wire.endTransmission();
  Wire.requestFrom(address, (byte)1);
  value = Wire.read();
  Wire.endTransmission();
  return value;
}

